// magicbit.js

(function(ext) {
    var device = null;
	
    var _rxBuf = [];

    // Sensor states:
    var ports = {
        Port1: 1,
        Port2: 2,
        Port3: 3,
        Port4: 4,
        Port5: 5,
        Port6: 6,
        Port7: 7,
        Port8: 8,
		M1:9,
		M2:10
    };
	var slots = {
		Slot1:1,
		Slot2:2
	};
	var switchStatus = {
		On:1,
		Off:0
	};
	var levels = {
		HIGH:1,
		LOW:0
	};
	var axis = {
		'X-Axis':1,
		'Y-Axis':2,
		'Z-Axis':3
	}
	var tones ={"B0":31,"C1":33,"D1":37,"E1":41,"F1":44,"G1":49,"A1":55,"B1":62,
			"C2":65,"D2":73,"E2":82,"F2":87,"G2":98,"A2":110,"B2":123,
			"C3":131,"D3":147,"E3":165,"F3":175,"G3":196,"A3":220,"B3":247,
			"C4":262,"D4":294,"E4":330,"F4":349,"G4":392,"A4":440,"B4":494,
			"C5":523,"D5":587,"E5":659,"F5":698,"G5":784,"A5":880,"B5":988,
			"C6":1047,"D6":1175,"E6":1319,"F6":1397,"G6":1568,"A6":1760,"B6":1976,
			"C7":2093,"D7":2349,"E7":2637,"F7":2794,"G7":3136,"A7":3520,"B7":3951,
	"C8":4186,"D8":4699};
	var beats = {"Half":500,"Quarter":250,"Eighth":125,"Whole":1000,"Double":2000,"Zero":0};
	var values = {};
	var indexs = [];
	
	var startTimer = 0;
	var versionIndex = 0xFA;
	
	var colors = {"Yellow":17,"Blue":33,"Green":32,"Red":16};
	var modes = {"ON":1,"OFF":0};
	
	var motortypes = {"A":1, "B":0};
	var RPMs = {"10":1,"20":2,"30":3,"40":4,"50":5};
	var directions = {"Forward":1,"Backward":0};
	
	var sizes = {"small":1,"medium":2,"large":3};
	
	var shapes = {"Circle":1,"Square":2,"Rectangle":3,"Triangle":4,"Filled Rectangle":5,"Filled Circle":6,"Filled Triangle":8,"Round Rectangle":9,"Heart":7};
	
	var humituretypes ={"humidity(%)":1,"temperature(°C )":0};
	
	var pwmvalues = {"0":1,"50":2,"100":3,"150":4,"255":5};
	
	var inbuiltsensors = {"ldr":36,"potentiometer":39,"left button":35,"right button":34,};
	
	var smilies = {"smile":1,"sad":2,"angry":3,"cool":4,"heart eyes":5};
	
 	var servovalues = {"0":0,"45":1,"90":2,"135":3,"180":4};
	
    ext.resetAll = function(){
    	device.send([0xff, 0x55, 2, 0, 4]);
    };
	
	ext.runArduino = function(){
		responseValue();
	};
	
	ext.getLDR = function(nextID,pin) {
		var deviceId = 27;
		getPackage(nextID,deviceId,36);
    };
	ext.runDigital = function(pin,level) {
        runPackage(30,pin,typeof level=="string"?levels[level]:new Number(level));
    };
    ext.runPwm = function(pin,pwm) {
        runPackage(32,pin,pwm);
    };
	ext.runTone = function(pin,tone,beat){
		runPackage(34,pin,short2array(typeof tone=="number"?tone:tones[tone]),short2array(typeof beat=="number"?beat:beats[beat]));
	};
	
	ext.setLED = function(pin,color,mode){
		runPackage(28,pin,short2array(typeof color=="number"?color:colors[color]),short2array(typeof mode=="number"?mode:modes[mode]));
	};
	
	ext.runMotor = function(pin,motortype,direction,r){

		runPackage(29,pin,short2array(typeof motortype=="number"?motortype:motortypes[motortype]),short2array(typeof direction=="number"?direction:directions[direction]),r);
	};
	
// Display
	ext.setText = function(size,x,y,message){
		runPackage(35,short2array(typeof size=="number"?size:sizes[size]),x,y,string2array(message+'\n'));
	};	
	
//Clear Display	
	
	ext.clearDisplay = function(){
		runPackage(39);
	};	
//Draw Shapes
	ext.drawShape = function(size,shape){
		runPackage(38,short2array(typeof size=="number"?size:sizes[size]),short2array(typeof shape=="number"?shape:shapes[shape]));
	};	
	
//Draw Smilies
	ext.drawSmile = function(pin,smile){
		runPackage(27,pin,short2array(typeof smile=="number"?smile:smilies[smile]));
	};		
	
	ext.runServo = function(pin, angle){
		runPackage(33,pin,angle);
	};
	ext.resetTimer = function(){
		startTimer = new Date().getTime();
		responseValue();
	};
	ext.getDigital = function(nextID,pin){
		var deviceId = 30;
		getPackage(nextID,deviceId,pin);
	};
	ext.getAnalog = function(nextID,pin) {
		var deviceId = 31;
		getPackage(nextID,deviceId,pin);
    };
	
	
	ext.getPulse = function(nextID,pin,timeout) {
		var deviceId = 37;
		getPackage(nextID,deviceId,pin,short2array(timeout));
    };
	ext.getUltrasonicArduino = function(nextID,trig,echo){
		var deviceId = 36;
		getPackage(nextID,deviceId,trig,echo);
	}
	ext.getTimer = function(nextID){
		if(startTimer==0){
			startTimer = new Date().getTime();
		}
		responseValue(nextID,new Date().getTime()-startTimer);
	}


//Humiture	

	ext.getHumiture = function(nextID,pin,humituretype) {
		var deviceId = 53;
		getPackage(nextID,deviceId,pin,short2array(typeof humituretype=="number"?humituretype:humituretypes[humituretype]));
    };	
//Inbuilt Sensors
	ext.getInbuiltSensor = function(nextID,inbuiltsensor) {
		var deviceId = 55;
		getPackage(nextID,deviceId,short2array(typeof inbuiltsensor=="number"?inbuiltsensor:inbuiltsensors[inbuiltsensor]));
    };	

//RGB	
	
		ext.setRGB = function(pin,r,g,b){
		runPackage(54,pin,r,g,b);
	};	
//PWM
	
	ext.setPWM = function(pin,pwm) {
        runPackage(52,pin,pwm);
    };

	function sendPackage(argList, type){
		var bytes = [0xff, 0x55, 0, 0, type];
		for(var i=0;i<argList.length;++i){
			var val = argList[i];
			if(val.constructor == "[class Array]"){
				bytes = bytes.concat(val);
			}else{
				bytes.push(val);
			}
		}
		bytes[2] = bytes.length - 3;
		device.send(bytes);
	}
	
	function runPackage(){
		sendPackage(arguments, 2);
	}
	function getPackage(){
		var nextID = arguments[0];
		Array.prototype.shift.call(arguments);
		sendPackage(arguments, 1);
	}

    var inputArray = [];
	var _isParseStart = false;
	var _isParseStartIndex = 0;
    function processData(bytes) {
		var len = bytes.length;
		if(_rxBuf.length>30){
			_rxBuf = [];
		}
		for(var index=0;index<bytes.length;index++){
			var c = bytes[index];
			_rxBuf.push(c);
			if(_rxBuf.length>=2){
				if(_rxBuf[_rxBuf.length-1]==0x55 && _rxBuf[_rxBuf.length-2]==0xff){
					_isParseStart = true;
					_isParseStartIndex = _rxBuf.length-2;
				}
				if(_rxBuf[_rxBuf.length-1]==0xa && _rxBuf[_rxBuf.length-2]==0xd&&_isParseStart){
					_isParseStart = false;
					
					var position = _isParseStartIndex+2;
					var extId = _rxBuf[position];
					position++;
					var type = _rxBuf[position];
					position++;
					//1 byte 2 float 3 short 4 len+string 5 double
					var value;
					switch(type){
						case 1:{
							value = _rxBuf[position];
							position++;
						}
							break;
						case 2:{
							value = readFloat(_rxBuf,position);
							position+=4;
							if(value<-255||value>1023){
								value = 0;
							}
						}
							break;
						case 3:{
							value = readInt(_rxBuf,position,2);
							position+=2;
						}
							break;
						case 4:{
							var l = _rxBuf[position];
							position++;
							value = readString(_rxBuf,position,l);
						}
							break;
						case 5:{
							value = readDouble(_rxBuf,position);
							position+=4;
						}
							break;
						case 6:
							value = readInt(_rxBuf,position,4);
							position+=4;
							break;
					}
					if(type<=6){
						responseValue(extId,value);
					}else{
						responseValue();
					}
					_rxBuf = [];
				}
			} 
		}
    }
	function readFloat(arr,position){
		var f= [arr[position],arr[position+1],arr[position+2],arr[position+3]];
		return parseFloat(f);
	}
	function readInt(arr,position,count){
		var result = 0;
		for(var i=0; i<count; ++i){
			result |= arr[position+i] << (i << 3);
		}
		return result;
	}
	function readDouble(arr,position){
		return readFloat(arr,position);
	}
	function readString(arr,position,len){
		var value = "";
		for(var ii=0;ii<len;ii++){
			value += String.fromCharCode(_rxBuf[ii+position]);
		}
		return value;
	}
    function appendBuffer( buffer1, buffer2 ) {
        return buffer1.concat( buffer2 );
    }

    // Extension API interactions
    var potentialDevices = [];
    ext._deviceConnected = function(dev) {
        potentialDevices.push(dev);

        if (!device) {
            tryNextDevice();
        }
    }

    function tryNextDevice() {
        // If potentialDevices is empty, device will be undefined.
        // That will get us back here next time a device is connected.
        device = potentialDevices.shift();
        if (device) {
            device.open({ stopBits: 0, bitRate: 115200, ctsFlowControl: 0 }, deviceOpened);
        }
    }

    var watchdog = null;
    function deviceOpened(dev) {
        if (!dev) {
            // Opening the port failed.
            tryNextDevice();
            return;
        }
        device.set_receive_handler('arduino',processData);
    };

    ext._deviceRemoved = function(dev) {
        if(device != dev) return;
        device = null;
    };

    ext._shutdown = function() {
        if(device) device.close();
        device = null;
    };

    ext._getStatus = function() {
        if(!device) return {status: 1, msg: 'Arduino disconnected'};
        if(watchdog) return {status: 1, msg: 'Probing for Arduino'};
        return {status: 2, msg: 'Arduino connected'};
    }

    var descriptor = {};
	ScratchExtensions.register('Arduino', descriptor, ext, {type: 'serial'});
})({});

//	["w", "Display %s of font size %d.sizes at x:%n y:%n","runIR", "text", "small", "0", "0",
//		{"encode":"{s0}","setup":"ledMtx_{0}.setColorIndex(1);\nledMtx_{0}.setBrightness(6);\n","inc":"","def":"MeLEDMatrix ledMtx_{0}({0});\n","work":"ledMtx_{0}.drawStr({1},{2}+7,{3});\n","loop":""}],
	// ext.runIR = function(message){
		// runPackage(35,string2array(message));
	// };	
//			{"encode":"{d0}","setup":"pinMode(YellowLED,OUTPUT);\n","inc":"","def":" #define YellowLED  13 ","work":"digitalWrite(YellowLED,{1});\ndelay({100});\ndigitalWrite(YellowLED,LOW);\ndelay({100});","loop":""}],	